--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.MovingView = {}

local function _RestoreWindowPosition(frame)
	local save = frame.profile.windowPosition
	if (save) then
		if (save.point) then
			frame:ClearAllPoints()
			frame:SetPoint(save.point, UIParent, save.relativePoint, save.x, save.y)
		end
	end
end

local function _SaveWindowPosition(frame)
	if (not frame.profile.windowPosition) then
		frame.profile.windowPosition = {}
	end
	local save = frame.profile.windowPosition
	local point, relativeTo, relativePoint, x, y = frame:GetPoint(1)
	save.point = point
	save.relativePoint = relativePoint
	save.x = math.floor(x + 0.5)
	save.y = math.floor(y + 0.5)
end

local function _GetMovable(frame)
	if (frame:IsMovable()) then
		return frame
	end
	if (frame:GetParent()) then
		return _GetMovable(frame:GetParent())
	end
end

local function _StopMovingOrSizing(frame)
	local parent = _GetMovable(frame)
	if (parent) then
		parent:StopMovingOrSizing()
		if (parent.SaveWindowPosition) then
			parent:SaveWindowPosition()
		end
	end
end

local function _StartMoving(frame)
	local parent = _GetMovable(frame)
	if (parent) then
		parent:StartMoving() 
	end
end

local function RegisterChildFrame(childFrame)
	childFrame:SetScript("OnMouseDown", _StartMoving)
	childFrame:SetScript("OnMouseUp", _StopMovingOrSizing)
end

local function RegisterFrame(frame)
	local name = frame:GetName()
	assert(name)
	if (name) then
		if (not petbm.PetBattleMaster.db.profile.frames[name]) then
			petbm.PetBattleMaster.db.profile.frames[name] = {}
		end
		frame.profile = petbm.PetBattleMaster.db.profile.frames[name]
		frame.SaveWindowPosition = _SaveWindowPosition
		frame.RestoreWindowPosition = _RestoreWindowPosition
	end
end

petbm.MovingView.RegisterChildFrame = RegisterChildFrame
petbm.MovingView.RegisterFrame = RegisterFrame